
/* noload.c by Peter Shipley */

/* Fri Jan 27 03:02:59 PST 1995 */
/* this program disables and open and ioctl of /dev/vd */
/* thus blocking modload from funtioning. */

/* inspired by a device driver by der Mouse <mouse@Collatz.McRCIM.McGill.EDU> */

#include <stdio.h>
#include <kvm.h>
#include <fcntl.h>
#include <nlist.h>
#include <sys/conf.h>

static struct nlist nl[] = {
    { "_cdevsw" },	
#define CDEVSW 0
    { "_vdopen" },
#define VDOPEN 1
    { "_vdclose" },
#define VDCLOSE 2
    { "_nodev" },
#define NODEV 3
    { "" },
};

#define nlsize (sizeof (nl) / sizeof (struct nlist))

#define	VD 57

static char *kmemf, *swapf, *nlistf;
static kvm_t   *kvmp;

static struct cdevsw cd;
static struct cdevsw *cd_p;

extern errno;

int printf();
int fprintf();
void exit();

static debug=0;


int
main(ac, av)
int ac;
char *av[];
{
int i;

    /* open the kmem device */
    kvmp = kvm_open(nlistf, kmemf, swapf, O_RDWR, av[0]);

    /* if kvm_open had failed it would have printed at error string for us */
    if(kvmp == NULL) {
	exit(1);
    }

    /* get the name list from the kernal */
    i = kvm_nlist(kvmp, nl);

    /* test that we obtained the namelist we wanted */
    if ( i > 0 ) {
	(void) fprintf(stderr,
	    "%s: kvm_nlist failed to read all symbols, aborting...\n",
	    av[0]);
	exit(1);
    }


    if (debug)  {
	(void) printf("n_name = %s  n_type=%x n_value=%x\n",
		nl[CDEVSW].n_name, nl[CDEVSW].n_type, nl[CDEVSW].n_value);
	(void) printf("n_name = %s  n_type=%x n_value=%x\n",
		nl[VDOPEN].n_name, nl[VDOPEN].n_type, nl[VDOPEN].n_value);
	(void) printf("n_name = %s  n_type=%x n_value=%x\n",
		nl[NODEV].n_name, nl[NODEV].n_type, nl[NODEV].n_value);
    }

    /*  calc the address the the 57'th array index */
    cd_p = &( ((struct cdevsw *) nl[0].n_value)[VD]);

    if(debug)  {
	(void) printf("%x %x\n", (int) nl[0].n_value, (int) cd_p);
    }

    /* read in the 57'th index if cdevsw */
    (void) kvm_read(kvmp, cd_p, &cd, sizeof(struct cdevsw));

    /* test that we got the right one */
    if( (caddr_t) cd.d_close != (caddr_t) nl[VDCLOSE].n_value ) {
	(void) fprintf(stderr,
	    "%s: Error loadable modules interface driver no at index %d, aborting...\n",
	    av[0], VD);
	exit(1);
    }

    /* test that we have not done this already */
    if( (caddr_t) cd.d_open == (caddr_t) nl[NODEV].n_value ) {
	(void) fprintf(stderr,
	    "%s: loadable modules interface driver has already been disabled, exiting..\n",
	    av[0]);
	exit(1);
    }

    cd.d_open = (caddr_t) nl[NODEV].n_value;
    cd.d_ioctl = (caddr_t) nl[NODEV].n_value;

    /* update the entery in the character device table */
    i = kvm_write(kvmp, cd_p, &cd, sizeof(struct cdevsw));

    /* report the sucess of the write */
    if (i == sizeof(struct cdevsw) ) {
	(void) printf("%s: loadable modules interface driver is now disabled\n",
	    av[0]);
    } else {
	(void) printf("%s: write error occored while updating the character device table\n",
	    av[0]);
    }

    /* it is safe to ignore the result of this operation */
    (void) kvm_close(kvmp);

    exit(0);

}

